// -*- C++ -*-
/*!
 * @file  coord_trans.cpp * @brief coordinate transformation * $Date$ 
 *
 * $Id$ 
 */
#include "coord_trans.h"

// Module specification
// <rtc-template block="module_spec">
static const char* coord_trans_spec[] =
  {
    "implementation_id", "coord_trans",
    "type_name",         "coord_trans",
    "description",       "coordinate transformation",
    "version",           "1.0",
    "vendor",            "Takashi Suehiro, AIST",
    "category",          "coord_trans",
    "activity_type",     "SPORADIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    ""
  };
// </rtc-template>

coord_trans::coord_trans(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_inIn("in", m_in),
    m_outOut("out", m_out),
    m_ComCtPort("ComCt")

    // </rtc-template>
{
  m_ct = new ExecCt(this);
  m_inIn.setOnWrite(m_ct);
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("in", m_inIn);

  // Set OutPort buffer
  registerOutPort("out", m_outOut);

  // Set service provider to Ports
  m_ComCtPort.registerProvider("com_ct", "ComCt", m_com_ct);

  // Set service consumers to Ports

  // Set CORBA Service Ports
  registerPort(m_ComCtPort);

  // </rtc-template>

  m_out.data.length(12);

}

coord_trans::~coord_trans()
{
}


RTC::ReturnCode_t coord_trans::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable

  // </rtc-template>
  act_flag = 0;
  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t coord_trans::onFinalize()
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t coord_trans::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t coord_trans::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
RTC::ReturnCode_t coord_trans::onActivated(RTC::UniqueId ec_id)
{
  act_flag = 1;
  return RTC::RTC_OK;
}
RTC::ReturnCode_t coord_trans::onDeactivated(RTC::UniqueId ec_id)
{
  act_flag = 0;
  return RTC::RTC_OK;
}
RTC::ReturnCode_t coord_trans::onExecute(RTC::UniqueId ec_id)
{
  usleep(10000);
  return RTC::RTC_OK;
}
/*
RTC::ReturnCode_t coord_trans::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
RTC::ReturnCode_t coord_trans::onError(RTC::UniqueId ec_id)
{
  act_flag = 0;
  return RTC::RTC_OK;
}
RTC::ReturnCode_t coord_trans::onReset(RTC::UniqueId ec_id)
{
  act_flag = 0;
  return RTC::RTC_OK;
}
/*
RTC::ReturnCode_t coord_trans::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t coord_trans::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


extern "C"
{
 
  void coord_transInit(RTC::Manager* manager)
  {
    RTC::Properties profile(coord_trans_spec);
    manager->registerFactory(profile,
                             RTC::Create<coord_trans>,
                             RTC::Delete<coord_trans>);
  }
  
};



