// -*- C++ -*-
/*!
 * @file  coord_trans.h * @brief coordinate transformation * @date  $Date$ 
 *
 * $Id$ 
 */
#ifndef COORD_TRANS_H
#define COORD_TRANS_H

#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>

// Service implementation headers
// <rtc-template block="service_impl_h">
#include "com_ctSVC_impl.h"

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>

class ExecCt;

using namespace RTC;

class coord_trans  : public RTC::DataFlowComponentBase
{
  friend class ExecCt;

 public:
  coord_trans(RTC::Manager* manager);
  ~coord_trans();

  // The initialize action (on CREATED->ALIVE transition)
  // formaer rtc_init_entry() 
 virtual RTC::ReturnCode_t onInitialize();

  // The finalize action (on ALIVE->END transition)
  // formaer rtc_exiting_entry()
  // virtual RTC::ReturnCode_t onFinalize();

  // The startup action when ExecutionContext startup
  // former rtc_starting_entry()
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  // The shutdown action when ExecutionContext stop
  // former rtc_stopping_entry()
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  // The activated action (Active state entry action)
  // former rtc_active_entry()
  virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  // The deactivated action (Active state exit action)
  // former rtc_active_exit()
  virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  // The execution action that is invoked periodically
  // former rtc_active_do()
  virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  // The aborting action when main logic error occurred.
  // former rtc_aborting_entry()
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  // The error action in ERROR state
  // former rtc_error_do()
  virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  // The reset action that is invoked resetting
  // This is same but different the former rtc_init_entry()
  virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  // The state update action that is invoked after onExecute() action
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  // The action that is invoked when execution context's rate is changed
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // Configuration variable declaration
  // <rtc-template block="config_declare">
  
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  TimedDoubleSeq m_in;
  InPort<TimedDoubleSeq> m_inIn;

  // </rtc-template>

  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  TimedDoubleSeq m_out;
  OutPort<TimedDoubleSeq, NullBuffer> m_outOut;

  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  RTC::CorbaPort m_ComCtPort;

  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  ComCtSVC_impl m_com_ct;

  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">

  // </rtc-template>

 private:
  int act_flag;
  ExecCt *m_ct;
  int dummy;

};

class ExecCt : public RTC::OnWrite<RTC::TimedDoubleSeq>
{
  coord_trans* ct;
public:
  ExecCt(coord_trans* comp) : ct(comp) {};
  void operator()(const RTC::TimedDoubleSeq& value)
  {
    int i,j;
    FRAME in_frm;
    FRAME out_frm;
    if(ct->act_flag==1) {   // controlled by onActivated and onDeactivated
      //    m_thIn.read();
      for(i=0;i<3;i++) {
	in_frm.vtr[i]=value.data[9+i];
	for(j=0;j<3;j++) {
	  in_frm.mtr[i][j]=value.data[3*i+j];
	}
      }
      //      std::cout << "in: " << value.data[0] ;
      //      std::cout << "    " << value.data[1] ;
      //      std::cout << "    " << value.data[2] ;
      //      std::cout << std::endl;
      f_mul(ct->m_com_ct.trans_frm,in_frm,out_frm);    
      //      mv_mul(ct->m_com_ct.trans_frm.mtr,in_frm.vtr,out_frm.vtr);
      //      v_add(ct->m_com_ct.trans_frm.vtr,out_frm.vtr,out_frm.vtr);
      //      m_mul(ct->m_com_ct.trans_frm.mtr,in_frm.mtr,out_frm.mtr);
      for(i=0;i<3;i++) {
	for(j=0;j<3;j++) {
	  ct->m_out.data[3*i+j]=out_frm.mtr[i][j];
	}
	ct->m_out.data[9+i]=out_frm.vtr[i];
      }
      //      std::cout << "out:" << ct->m_out.data[0] ;
      //      std::cout << "    " << ct->m_out.data[1] ;
      //      std::cout << "    " << ct->m_out.data[2] ;
      //      std::cout << std::endl;
      ct->m_outOut.write();
    }
  };
};

extern "C"
{
  void coord_transInit(RTC::Manager* manager);
};

#endif // COORD_TRANS_H

