// -*- C++ -*-
/*!
 * @file  frm_ctrl.cpp
 * @brief motion control in task coordinate
 * $Date$
 *
 * $Id$
 */

#include "frm_ctrl.h"

// Module specification
// <rtc-template block="module_spec">
static const char* frm_ctrl_spec[] =
  {
    "implementation_id", "frm_ctrl",
    "type_name",         "frm_ctrl",
    "description",       "motion control in task coordinate",
    "version",           "1.0",
    "vendor",            "Takashi Suehiro, AIST",
    "category",          "frm_ctrl",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables

    ""
  };
// </rtc-template>

frm_ctrl::frm_ctrl(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_ref_frmIn("ref_frm", m_ref_frm),
    m_cur_frmIn("cur_frm", m_cur_frm),
    m_velOut("vel", m_vel),
    m_stateOut("state", m_state),
    m_ComFrmCtrlPort("ComFrmCtrl"),
    // </rtc-template>
	dummy(0)
{
  m_eref = new ExecRef(this);
  m_ecur = new ExecCur(this);
  m_ref_frmIn.setOnWrite(m_eref);
  m_cur_frmIn.setOnWrite(m_ecur);
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("ref_frm", m_ref_frmIn);
  registerInPort("cur_frm", m_cur_frmIn);
  
  // Set OutPort buffer
  registerOutPort("vel", m_velOut);
  registerOutPort("state", m_stateOut);
  
  // Set service provider to Ports
  m_ComFrmCtrlPort.registerProvider("com_frm_ctrl", "ComFrmCtrl", m_com_frm_ctrl);
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  registerPort(m_ComFrmCtrlPort);

  // </rtc-template>

  //
  // out port buffer area
  //

  m_vel.data.length(6);  // vel of xyzabc


  // default
  // k_gain = 1.0, means time constant is 1 sec.
  // v_max = 200[mm/sec]
  // k_pos_rot = 200[mm], means that sizes of objects are about this size.
  m_com_frm_ctrl.set_param(1.0, 200, 200);
  m_com_frm_ctrl.settle = 1;
  m_com_frm_ctrl.state = 0;

}

frm_ctrl::~frm_ctrl()
{
}


RTC::ReturnCode_t frm_ctrl::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  act_flag = 0;
  m_ref_flag = 0;
  m_cur_flag = 0;
  first_flag = 1;
  // </rtc-template>
  return RTC::RTC_OK;
}



/*
RTC::ReturnCode_t frm_ctrl::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t frm_ctrl::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t frm_ctrl::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
RTC::ReturnCode_t frm_ctrl::onActivated(RTC::UniqueId ec_id)
{
  act_flag = 1;
  m_ref_flag = 0;
  m_cur_flag = 0;
  first_flag = 1;
  return RTC::RTC_OK;
  /*
  int i;

  state_old = m_com_frm_ctrl.state;
  i = 0;
  std::cout << "checking input" << std::endl;
  for(i=0;i<1000;i++) {
    if((not m_ref_frmIn.isEmpty()) && (not m_cur_frmIn.isEmpty())) {
      return RTC::RTC_OK;
      std::cout << "ready" << std::endl;
    }
    usleep(1000);
  }
  std::cout << "lack of input" << std::endl;
  return RTC::RTC_ERROR;
  */
}

RTC::ReturnCode_t frm_ctrl::onDeactivated(RTC::UniqueId ec_id)
{
  act_flag = 0;
  m_ref_flag = 0;
  m_cur_flag = 0;
  first_flag = 1;
  return RTC::RTC_OK;
}

RTC::ReturnCode_t frm_ctrl::onExecute(RTC::UniqueId ec_id)
{
  /*
  int i,j;
  VECTOR p_vel, r_vel;
  int new_flag;

  new_flag = 0;
  if(not m_ref_frmIn.isEmpty()) {
    new_flag = 1;
    m_ref_frmIn.read();
    for(i=0;i<3;i++) {
      for(j=0;j<3;j++) {
	ref_frm.mtr[i][j]=m_ref_frm.data[3*i+j];
      }
      ref_frm.vtr[i]=m_ref_frm.data[9+i];
    }
    //    std::cout << " new ref  " ;
    //    std::cout << ref_frm.vtr[0] << " " ;
    //    std::cout << ref_frm.vtr[1] << " " ;
    //    std::cout << ref_frm.vtr[2] << " " ;
    //    std::cout << std::endl;
  }
  //  std::cout << ref_frm.vtr[0] << " " ;
  //  std::cout << ref_frm.vtr[1] << " " ;
  //  std::cout << ref_frm.vtr[2] << " " ;
  //  std::cout << std::endl;
  if(not m_cur_frmIn.isEmpty()) {
    m_cur_frmIn.read();
    new_flag = 1;
    for(i=0;i<3;i++) {
      for(j=0;j<3;j++) {
	cur_frm.mtr[i][j]=m_cur_frm.data[3*i+j];
      }
      cur_frm.vtr[i]=m_cur_frm.data[9+i];
    }
    //    std::cout << " new cur " ;
    //    std::cout << cur_frm.vtr[0] << " " ;
    //    std::cout << cur_frm.vtr[1] << " " ;
    //    std::cout << cur_frm.vtr[2] << " " ;
    //    std::cout << std::endl;
  }
  //  std::cout << cur_frm.vtr[0] << " " ;
  //  std::cout << cur_frm.vtr[1] << " " ;
  //  std::cout << cur_frm.vtr[2] << " " ;
  //  std::cout << std::endl;
  if(new_flag == 1) {
    m_com_frm_ctrl.calc_vel(ref_frm, cur_frm, p_vel, r_vel);
    for(i=0;i<3;i++) {
      m_vel.data[i]=p_vel[i];
      m_vel.data[i+3]=r_vel[i];
    }
    m_velOut.write();
    if(state_old != m_com_frm_ctrl.state) {
      //    std::cout << " old :" <<  state_old;
      state_old = m_com_frm_ctrl.state;
      //    std::cout << "  new :" <<  state_old << std::endl;
      m_state.data = state_old;
      m_stateOut.write();
    }
  }
  //  usleep(1000);
  */
  usleep(10000);
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t frm_ctrl::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

RTC::ReturnCode_t frm_ctrl::onError(RTC::UniqueId ec_id)
{
  act_flag = 0;
  m_ref_flag = 0;
  m_cur_flag = 0;
  first_flag = 1;
  return RTC::RTC_OK;
}

RTC::ReturnCode_t frm_ctrl::onReset(RTC::UniqueId ec_id)
{
  act_flag = 0;
  m_ref_flag = 0;
  m_cur_flag = 0;
  first_flag = 1;
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t frm_ctrl::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t frm_ctrl::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void frm_ctrlInit(RTC::Manager* manager)
  {
    RTC::Properties profile(frm_ctrl_spec);
    manager->registerFactory(profile,
                             RTC::Create<frm_ctrl>,
                             RTC::Delete<frm_ctrl>);
  }
  
};


