// -*- C++ -*-
/*!
 * @file  j_jacob.cpp * @brief j_jacob * $Date$ 
 *
 * $Id$ 
 */
#include "j_jacob.h"

// Module specification
// <rtc-template block="module_spec">
static const char* j_jacob_spec[] =
  {
    "implementation_id", "j_jacob",
    "type_name",         "j_jacob",
    "description",       "j_jacob",
    "version",           "1.0",
    "vendor",            "Takashi Suehiro, AIST",
    "category",          "coord_trans",
    "activity_type",     "SPORADIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    ""
  };
// </rtc-template>

j_jacob::j_jacob(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_velIn("vel", m_vel),
    m_y_jacobOut("y_jacob", m_y_jacob)

    // </rtc-template>
{
  m_evel=new ExecVel(this);
  m_velIn.setOnWrite(m_evel);
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("vel", m_velIn);

  // Set OutPort buffer
  registerOutPort("y_jacob", m_y_jacobOut);

  // Set service provider to Ports

  // Set service consumers to Ports

  // Set CORBA Service Ports

  // </rtc-template>
  m_y_jacob.data.length(56); // (1 + m) * n, where m = 7, n = 7
}

j_jacob::~j_jacob()
{
}


RTC::ReturnCode_t j_jacob::onInitialize()
{
  int i,j;
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable

  // </rtc-template>
  act_flag = 0;
  for (i=0;i<7;i++) {
    for(j=0;j<8;j++) {
      m_y_jacob.data[8*i+j]=0.0;
    }
    m_y_jacob.data[8*i+i+1]=1.0;
  }
  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t j_jacob::onFinalize()
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t j_jacob::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t j_jacob::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

RTC::ReturnCode_t j_jacob::onActivated(RTC::UniqueId ec_id)
{
  act_flag = 1;
  return RTC::RTC_OK;
}
RTC::ReturnCode_t j_jacob::onDeactivated(RTC::UniqueId ec_id)
{
  act_flag = 0;
  return RTC::RTC_OK;
}
RTC::ReturnCode_t j_jacob::onExecute(RTC::UniqueId ec_id)
{
  usleep(10000);
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t j_jacob::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
RTC::ReturnCode_t j_jacob::onError(RTC::UniqueId ec_id)
{
  act_flag = 0;
  return RTC::RTC_OK;
}
RTC::ReturnCode_t j_jacob::onReset(RTC::UniqueId ec_id)
{
  act_flag = 0;
  return RTC::RTC_OK;
}
/*
RTC::ReturnCode_t j_jacob::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t j_jacob::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


extern "C"
{
 
  void j_jacobInit(RTC::Manager* manager)
  {
    RTC::Properties profile(j_jacob_spec);
    manager->registerFactory(profile,
                             RTC::Create<j_jacob>,
                             RTC::Delete<j_jacob>);
  }
  
};



