// -*- C++ -*-
/*!
 * @file  pa10vel.h * @brief velocity controlled pa10 arm * @date  $Date$ 
 *
 * $Id$ 
 */
#ifndef PA10VEL_H
#define PA10VEL_H

#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <sys/time.h>
  //#include "pa10_library.h"
extern "C" {
#include "pa10_library.h"
}


// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>

class VelCB;

using namespace RTC;

class pa10vel  : public RTC::DataFlowComponentBase
{
  friend class VelCB;
 public:
  pa10vel(RTC::Manager* manager);
  ~pa10vel();

  // The initialize action (on CREATED->ALIVE transition)
  // formaer rtc_init_entry() 
  virtual RTC::ReturnCode_t onInitialize();

  // The finalize action (on ALIVE->END transition)
  // formaer rtc_exiting_entry()
  virtual RTC::ReturnCode_t onFinalize();

  // The startup action when ExecutionContext startup
  // former rtc_starting_entry()
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  // The shutdown action when ExecutionContext stop
  // former rtc_stopping_entry()
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  // The activated action (Active state entry action)
  // former rtc_active_entry()
  virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  // The deactivated action (Active state exit action)
  // former rtc_active_exit()
  virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  // The execution action that is invoked periodically
  // former rtc_active_do()
  virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  // The aborting action when main logic error occurred.
  // former rtc_aborting_entry()
  virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  // The error action in ERROR state
  // former rtc_error_do()
  virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  // The reset action that is invoked resetting
  // This is same but different the former rtc_init_entry()
  virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  // The state update action that is invoked after onExecute() action
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  // The action that is invoked when execution context's rate is changed
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // Configuration variable declaration
  // <rtc-template block="config_declare">
  
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  TimedDoubleSeq m_vel;
  InPort<TimedDoubleSeq> m_velIn;

  // </rtc-template>

  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  TimedDoubleSeq m_th;
  OutPort<TimedDoubleSeq, NullBuffer> m_thOut;

  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">

  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">

  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">

  // </rtc-template>

 private:

  //  TimeMeasure tim;
  RTC::Time in_tm;
  double ref_vel[7];
  VelCB* m_velcb;

  pa10_info *pa10;         // for pa10_library.h
  pa10_state *pa10_st;      // for pa10_library.h

};

class VelCB :public RTC::OnWrite<RTC::TimedDoubleSeq>
{
  pa10vel* m_pa10vel;
public :
  VelCB(pa10vel* comp) : m_pa10vel(comp) {};
  void operator() (const RTC::TimedDoubleSeq& value)
  {
    int i;
    timeval time_val;

    gettimeofday(&time_val, NULL);
    m_pa10vel -> in_tm.sec = time_val.tv_sec;
    m_pa10vel -> in_tm.nsec = time_val.tv_usec * 1000;
    for(i=0;i<7;i++) {
      m_pa10vel -> ref_vel[i]=value.data[i];
    }
  };
};


extern "C"
{
  void pa10velInit(RTC::Manager* manager);
};

#endif // PA10VEL_H

