// -*- C++ -*-
/*!
 * @file  tr_jacob.h * @brief tr_jacob * @date  $Date$ 
 *
 * $Id$ 
 */
#ifndef TR_JACOB_H
#define TR_JACOB_H

#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>

// Service implementation headers
// <rtc-template block="service_impl_h">
#include "com_ctSVC_impl.h"

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>

class ExecVel;
class ExecJacob;

using namespace RTC;

class tr_jacob  : public RTC::DataFlowComponentBase
{
  friend class ExecVel;
  friend class ExecJacob;
 public:
  tr_jacob(RTC::Manager* manager);
  ~tr_jacob();

  // The initialize action (on CREATED->ALIVE transition)
  // formaer rtc_init_entry() 
 virtual RTC::ReturnCode_t onInitialize();

  // The finalize action (on ALIVE->END transition)
  // formaer rtc_exiting_entry()
  // virtual RTC::ReturnCode_t onFinalize();

  // The startup action when ExecutionContext startup
  // former rtc_starting_entry()
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  // The shutdown action when ExecutionContext stop
  // former rtc_stopping_entry()
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  // The activated action (Active state entry action)
  // former rtc_active_entry()
  virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  // The deactivated action (Active state exit action)
  // former rtc_active_exit()
  virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  // The execution action that is invoked periodically
  // former rtc_active_do()
  virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  // The aborting action when main logic error occurred.
  // former rtc_aborting_entry()
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  // The error action in ERROR state
  // former rtc_error_do()
  virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  // The reset action that is invoked resetting
  // This is same but different the former rtc_init_entry()
  virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  // The state update action that is invoked after onExecute() action
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  // The action that is invoked when execution context's rate is changed
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // Configuration variable declaration
  // <rtc-template block="config_declare">
  
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  TimedDoubleSeq m_vel;
  InPort<TimedDoubleSeq> m_velIn;
  TimedDoubleSeq m_jacob;
  InPort<TimedDoubleSeq> m_jacobIn;

  // </rtc-template>

  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  TimedDoubleSeq m_y_jacob;
  OutPort<TimedDoubleSeq, NullBuffer> m_y_jacobOut;

  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  RTC::CorbaPort m_ComCtPort;

  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  ComCtSVC_impl m_com_ct;

  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">

  // </rtc-template>

 private:
  ExecVel* m_evel;
  ExecJacob* m_ejacob;
  int act_flag, m_vel_flag, m_jacob_flag;
  int dummy;
};

class ExecVel : public RTC::OnWrite<RTC::TimedDoubleSeq>
{
  tr_jacob* m_trjcb;
public :
  ExecVel(tr_jacob* comp) : m_trjcb(comp) {};
  void operator()(const RTC::TimedDoubleSeq& value) {
    int i;
    if(m_trjcb->act_flag) {
      //    std::cout << " vel" << std::endl;
      m_trjcb->m_vel_flag = 1;
      for(i=0;i<6;i++) {
	m_trjcb->m_y_jacob.data[i*8]=value.data[i];
      }
      if(m_trjcb->m_jacob_flag == 1) {
	//	std::cout << "out:" << m_trjcb->m_y_jacob.data[1] ;
	//	std::cout << "    " << m_trjcb->m_y_jacob.data[2] ;
	//	std::cout << "    " << m_trjcb->m_y_jacob.data[3] ;
	//	std::cout << std::endl;
	m_trjcb->m_y_jacobOut.write();
	//    std::cout << " write j_list" << std::endl;
      }
    }
  };
};



class ExecJacob : public RTC::OnWrite<RTC::TimedDoubleSeq>
{
  tr_jacob* m_trjcb;
  VECTOR t_tmp,r_tmp;
public :
  ExecJacob(tr_jacob* comp) : m_trjcb(comp) {};
  void operator()(const RTC::TimedDoubleSeq& value) {
    int i,j;
    if(m_trjcb->act_flag) {
      //    std::cout << " vel" << std::endl;
      m_trjcb->m_jacob_flag = 1;
      //      std::cout << "in: " << value.data[0] ;
      //      std::cout << "    " << value.data[1] ;
      //      std::cout << "    " << value.data[2] ;
      //      std::cout << std::endl;
      for(j=0;j<7;j++) {
	for(i=0;i<3;i++) {
	  t_tmp[i]=value.data[i*7+j];
	  r_tmp[i]=value.data[(3+i)*7+j];
	}
	mv_mul(m_trjcb->m_com_ct.trans_frm.mtr,t_tmp,t_tmp);
	mv_mul(m_trjcb->m_com_ct.trans_frm.mtr,r_tmp,r_tmp);
	for(i=0;i<3;i++) {
	  m_trjcb->m_y_jacob.data[i*8+1+j]=t_tmp[i];
	  m_trjcb->m_y_jacob.data[(3+i)*8+1+j]=r_tmp[i];
	}
      }
    }
  };
};



extern "C"
{
  void tr_jacobInit(RTC::Manager* manager);
};

#endif // TR_JACOB_H

