// -*- C++ -*-
/*!
 * @file  vel_7dof.cpp
 * @brief simulator of velocity controlled 7dof arm
 * $Date$
 *
 * $Id$
 */

#include "vel_7dof.h"

// Module specification
// <rtc-template block="module_spec">
static const char* vel_7dof_spec[] =
  {
    "implementation_id", "vel_7dof",
    "type_name",         "vel_7dof",
    "description",       "simulator of velocity controlled 7dof arm",
    "version",           "1.0",
    "vendor",            "Takashi Suehiro, AIST",
    "category",          "vel_7dof",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables

    ""
  };
// </rtc-template>

vel_7dof::vel_7dof(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_velIn("vel", m_vel),
    m_thOut("th", m_th),
    
    // </rtc-template>
	dummy(0)
{
  m_velcb= new VelCB(this);
  m_velIn.setOnWrite(m_velcb);
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("vel", m_velIn);
  
  // Set OutPort buffer
  registerOutPort("th", m_thOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

vel_7dof::~vel_7dof()
{
}


RTC::ReturnCode_t vel_7dof::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  
  // </rtc-template>
  //  int error;
  int i;
  m_th.data.length(7);
  for(i=0;i<7;i++) {
    ref_vel[i]=0.0;
    th[i]=0.0;
  }
  return RTC::RTC_OK;
}

RTC::ReturnCode_t vel_7dof::onFinalize()
{
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t vel_7dof::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t vel_7dof::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t vel_7dof::onActivated(RTC::UniqueId ec_id)
{
  //  int error;
  int i;
  timeval time_val;

  //  std::cout << "baka 0" << std::endl;

  //  tim.new_time(n_tm.sec, n_tm.nsec);
  gettimeofday(&time_val, NULL);
  in_tm.sec = time_val.tv_sec;
  in_tm.nsec = time_val.tv_usec * 1000;
  o_tm.sec=in_tm.sec;
  o_tm.nsec=in_tm.nsec;
  //  std::cout << "baka 1" << std::endl;
  for(i=0;i<7;i++) {
    ref_vel[i]=0.0;
  }
  for(i=0;i<7;i++) {
    m_th.data[i]=th[i];
  }
  //  std::cout << "baka 3" << std::endl;
  m_thOut.write();
  //  std::cout << "baka 4" << std::endl;
  usleep(10000);
  //  std::cout << "baka 5" << std::endl;
  return RTC::RTC_OK;
}

RTC::ReturnCode_t vel_7dof::onDeactivated(RTC::UniqueId ec_id)
{
  int i;
  for(i=0;i<7;i++) {
    ref_vel[i]=0.0;
  }
  return RTC::RTC_OK;
}

//-------------------------------------------------------------
// helper functions
//-------------------------------------------------------------
float calc_tm_dif(RTC::Time tm1, RTC::Time tm2) {
  return (float)(tm1.sec - tm2.sec)
    + (float)(1000000000 + tm1.nsec - tm2.nsec)/ 1000000000.0 - 1.0;
}
//
float calc_decay(double d_ftm, double l_ftm, double t_cnst) {
  if(d_ftm > l_ftm) {
    return exp((l_ftm - d_ftm)/ t_cnst);
  } else {
    return 1.0;
  }
}
//--------------------------------------------------------
RTC::ReturnCode_t vel_7dof::onExecute(RTC::UniqueId ec_id)
{
  int i;
  timeval time_val;
  RTC::Time n_tm;
  double d_tm, di_tm, d_fact;
  double vel[7];

  //  std::cout << "aho 0" << std::endl;

  //  tim.new_time(n_tm.sec, n_tm.nsec);

  gettimeofday(&time_val, NULL);
  n_tm.sec = time_val.tv_sec;
  n_tm.nsec = time_val.tv_usec * 1000;
  d_tm = calc_tm_dif(n_tm, o_tm);  // delta time1
  di_tm = calc_tm_dif(n_tm, in_tm);  // delta time2
  //  d_fact = calc_decay(di_tm, 0.1, 1.0);  // apply decay
  d_fact = calc_decay(di_tm, 0.1, 0.2);  // apply decay
  o_tm.sec=n_tm.sec;
  o_tm.nsec=n_tm.nsec;
  for(i=0;i<7;i++) {
    vel[i]=d_fact*ref_vel[i];
    th[i] += vel[i]*d_tm;
    m_th.data[i]=th[i];
  }
  //  std::cout << "aho 7" << std::endl;
  m_thOut.write();
  //  std::cout << "aho 8" << std::endl;
  //  std::cout << m_th.data.length() << std::endl;
  usleep(10000);
  return RTC::RTC_OK;
}


RTC::ReturnCode_t vel_7dof::onAborting(RTC::UniqueId ec_id)
{
  int i;
  for(i=0;i<7;i++) {
    ref_vel[i]=0.0;
  }
  return RTC::RTC_OK;
}

RTC::ReturnCode_t vel_7dof::onError(RTC::UniqueId ec_id)
{
  int i;
  for(i=0;i<7;i++) {
    ref_vel[i]=0.0;
  }
  return RTC::RTC_OK;
}

RTC::ReturnCode_t vel_7dof::onReset(RTC::UniqueId ec_id)
{
  int i;
  for(i=0;i<7;i++) {
    ref_vel[i]=0.0;
  }
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t vel_7dof::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t vel_7dof::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{

  void vel_7dofInit(RTC::Manager* manager)
  {
    RTC::Properties profile(vel_7dof_spec);
    manager->registerFactory(profile,
                             RTC::Create<vel_7dof>,
                             RTC::Delete<vel_7dof>);
  }

};


